# Research Project Overview

## Introduction
This project is designed to conduct and analyze listening tests using modified webMUSHRA software. The project is structured into five main directories, each serving a specific purpose in the research workflow. Below is an overview of each directory and its contents.

## Directory Structure

### 1. Docs
- **Description:** Contains all the ethics documents concerning Human Research Ethics Committee (HREC), data management, and other ethical procedures.
- **Purpose:** These documents ensure that the correct ethical procedures are followed, which is crucial for the responsible research component of the project.
- **Contents:**
  - Ethics approval documents
  - Data management plans
  - Consent forms

### 2. Modified webMUSHRA Software
- **Description:** Contains the modified testing code used to conduct the actual listening tests.
- **Purpose:** This software is run with Docker (see the corresponding README for instructions) and includes configurations for paired_comparison and subjective_eval tests to obtain data from participants.
- **Contents:**
  - Modified webMUSHRA codebase
  - Configuration files for paired_comparison and subjective_eval tests
  - README with instructions on running the software with Docker

### 3. Results
- **Description:** Contains all the results obtained throughout the project, both in raw and processed form.
- **Purpose:** To store and organize the data collected from the listening tests. The processed_results folder contains the final results analyzed in the paper.
- **Contents:**
  - Raw results folders (`paired_results` & `subjective_results`)
  - Processed results folder (includes Excel files with in-depth statistical analysis described in the paper)

### 4. Samples
- **Description:** The audio samples used in the experiment.
- **Purpose:** To provide the necessary audio files for the listening tests. The Convolved folder contains each audio convolved with real and simulated Room Impulse Responses (RIRs) in their respective folders.
- **Contents:**
  - Original audio samples
  - Convolved audio samples
    - Real RIRs folder
    - Simulated RIRs folder

### 5. Utils
- **Description:** Contains all the scripts used in the research project for various purposes.
- **Purpose:** These scripts are used for generating sine sweeps, convolving and clipping the audios, and conducting some simple statistical analysis. Note that the in-depth analysis was done in Excel and can be found in the processed_results folder in Results.
- **Contents:**
  - Scripts for generating sine sweeps
  - Scripts for convolving and clipping audio files
  - Basic statistical analysis scripts

## Running the Software
Refer to the README in the `Modified webMUSHRA Software` directory for detailed instructions on how to run the software using Docker. The paired_comparison and subjective_eval configurations should be used to conduct the listening tests.

## Conclusion
This directory structure is designed to facilitate the smooth execution and analysis of the listening tests, ensuring that all necessary components are well-organized and reproducible. For any questions contact b.christensen@student.tudelft.nl
