"""
Script for obtaining the p-value for the paired comparison results. The current values represent the
results from the paired comparison testing, and running this script will output the p-value per audio sample type.
"""


from scipy.stats import binom_test

# Data for each combination of audio type and method
data = {
    'Classical': {'Pyroom': 20, 'Habets': 17},
    'Sine Sweep': {'Pyroom': 12, 'Habets': 18},
    'Speech': {'Pyroom': 24, 'Habets': 26}
}

# Total trials for each combination
total_trials = 27

# Perform binomial test for each combination
binomial_results = {}
for audio_type, methods in data.items():
    binomial_results[audio_type] = {}
    for method, correct in methods.items():
        p_value = binom_test(correct, total_trials, 0.5, alternative='greater')
        binomial_results[audio_type][method] = p_value

# Print results
for audio_type, methods in binomial_results.items():
    print(f"{audio_type}:")
    for method, p_value in methods.items():
        print(f"  {method}: p-value = {p_value:.6f}")

binomial_results

