"""
This file takes three files and clips them to be of the same length (of the shortest audio), and a bit shorter.
This was used because the audios needed to be of the same length to go into the test.
"""

from pydub import AudioSegment

def clip_to_shortest(audio_file1, audio_file2, audio_file3, output_file1, output_file2, output_file3):
    """
    This method clips the audio files, and outputs them to their respective output files.
    """

    # Load the audio files
    audio1 = AudioSegment.from_file(audio_file1)
    audio2 = AudioSegment.from_file(audio_file2)
    audio3 = AudioSegment.from_file(audio_file3)

    # Determine the length of the shorter audio
    min_length = min(len(audio3), min(len(audio1), len(audio2)))
    min_length*=0.9

    # Clip both audios to the length of the shorter one
    clipped_audio1 = audio1[:min_length]
    clipped_audio2 = audio2[:min_length]
    clipped_audio3 = audio3[:min_length]

    # Export the clipped audios to new files
    clipped_audio1.export(output_file1, format="wav")
    clipped_audio2.export(output_file2, format="wav")
    clipped_audio3.export(output_file3, format="wav")

    print(f"Clipped audio files saved as {output_file1} and {output_file2}")

# Filepaths are specified and the outuput paths are the same as I wanted to overwrite the audios with their clipped versions
filepath_1 = "insert/audio/path/here"
filepath_2 = "insert/audio/path/here"
filepath_3 = "insert/audio/path/here"
clip_to_shortest(filepath_1, filepath_2, filepath_3, filepath_1, filepath_2, filepath_3)
