"""
Generates a sine sweep. This was used for the sine sweep sample that was convolved with in the experiment.
"""

import numpy as np
from scipy.io.wavfile import write

def generate_sine_sweep(start_freq, end_freq, duration, sample_rate):
    """
    Generates a sine sweep with a starting frequency, an ending frequency, a duration and a sample rate specified.
    """
    t = np.linspace(0, duration, int(sample_rate * duration))
    sweep = np.sin(2 * np.pi * start_freq * (duration / np.log(end_freq / start_freq)) *
                   (np.exp(t * np.log(end_freq / start_freq) / duration) - 1))
    return sweep

# Parameters
start_freq = 20        # Start frequency of the sweep
end_freq = 4000     # End frequency of the sweep
duration = 5          # Duration of the sweep in seconds
sample_rate = 16000    # Sample rate in Hz

# Generate sine sweep
sine_sweep = generate_sine_sweep(start_freq, end_freq, duration, sample_rate)

# Normalize to 16-bit PCM range
sine_sweep = np.int16(sine_sweep / np.max(np.abs(sine_sweep)) * 32767)

# Save to a WAV file
write("insert/path/here", sample_rate, sine_sweep)
